/**************************************************************************//**
 * @file     ACMP.h
 * @version  V2.1
 * $Revision: 5 $
 * $Date: 14/02/11 7:24p $
 * @brief    M051 Series ACMP Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __ACMP_H__
#define __ACMP_H__

/*---------------------------------------------------------------------------------------------------------*/
/* Include related headers                                                                                 */
/*---------------------------------------------------------------------------------------------------------*/
#include "M051Series.h"


/** @addtogroup M051_FUNC M051 Function Interface
  * @{
  */

/** @addtogroup ACMP_FUNC ACMP Device Function Interface
  * @{
  */



/*---------------------------------------------------------------------------------------------------------*/
/* ACMP_CR constant definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define ACMP_CR_OUTPUT_INV           (1UL << 6) /*!< ACMP_CR setting for ACMP output inverse function. */
#define ACMP_CR_VNEG_BANDGAP         (1UL << 4) /*!< ACMP_CR setting for selecting band-gap voltage as the source of ACMP V-. */
#define ACMP_CR_VNEG_PIN             (0UL << 4) /*!< ACMP_CR setting for selecting the voltage of ACMP negative input pin as the source of ACMP V-. */
#define ACMP_CR_HYSTERESIS_EN        (1UL << 2) /*!< ACMP_CR setting for enabling the hysteresis function. */
#define ACMP_CR_INT_EN               (1UL << 1) /*!< ACMP_CR setting for enabling the interrupt function. */
#define ACMP_CR_ACMP_EN              (1UL << 0) /*!< ACMP_CR setting for enabling the ACMP. */

/*---------------------------------------------------------------------------------------------------------*/
/* ACMP_SR01 constant definitions                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
#define ACMP_SR01_ACMP1_OUTPUT_STATUS        (1UL << 3) /*!< ACMP_SR01 ACMP1 output status */
#define ACMP_SR01_ACMP0_OUTPUT_STATUS        (1UL << 2) /*!< ACMP_SR01 ACMP0 output status */
#define ACMP_SR01_ACMP1_INT_FLAG             (1UL << 1) /*!< ACMP_SR01 ACMP1 interrupt flag */
#define ACMP_SR01_ACMP0_INT_FLAG             (1UL << 0) /*!< ACMP_SR01 ACMP0 interrupt flag */

/*---------------------------------------------------------------------------------------------------------*/
/* ACMP_SR23 constant definitions                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
#define ACMP_SR23_ACMP3_OUTPUT_STATUS        (1UL << 3) /*!< ACMP_SR23 ACMP3 output status */
#define ACMP_SR23_ACMP2_OUTPUT_STATUS        (1UL << 2) /*!< ACMP_SR23 ACMP2 output status */
#define ACMP_SR23_ACMP3_INT_FLAG             (1UL << 1) /*!< ACMP_SR23 ACMP3 interrupt flag */
#define ACMP_SR23_ACMP2_INT_FLAG             (1UL << 0) /*!< ACMP_SR23 ACMP2 interrupt flag */

/*---------------------------------------------------------------------------------------------------------*/
/*  Define Macros and functions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/

/**
 * @brief      Enable ACMP output inverse function
 *
 * @param[in]  u32AcmpModule  Specify the analog comparator module
 *
 * @return     None
 *
 * @details    Enable ACMP output inverse function.
 */
static __INLINE void ACMP_EnableOutputInv(uint32_t u32AcmpModule)
{
    if(u32AcmpModule == 0)
        ACMP01->CR0 |= ACMP_CR_OUTPUT_INV;
    else if(u32AcmpModule == 1)
        ACMP01->CR1 |= ACMP_CR_OUTPUT_INV;
    else if(u32AcmpModule == 2)
        ACMP23->CR2 |= ACMP_CR_OUTPUT_INV;
    else
        ACMP23->CR3 |= ACMP_CR_OUTPUT_INV;
}

/**
 * @brief      Disable ACMP output inverse function
 *
 * @param[in]  u32AcmpModule  Specify the analog comparator module
 *
 * @return     None
 *
 * @details    Disable ACMP output inverse function.
 */
static __INLINE void ACMP_DisableOutputInv(uint32_t u32AcmpModule)
{
    if(u32AcmpModule == 0)
        ACMP01->CR0 &= (~ACMP_CR_OUTPUT_INV);
    else if(u32AcmpModule == 1)
        ACMP01->CR1 &= (~ACMP_CR_OUTPUT_INV);
    else if(u32AcmpModule == 2)
        ACMP23->CR2 &= (~ACMP_CR_OUTPUT_INV);
    else
        ACMP23->CR3 &= (~ACMP_CR_OUTPUT_INV);
}

/**
 * @brief      Select ACMP negative input source
 *
 * @param[in]  u32AcmpModule  Specify the analog comparator module
 *             u32NegSrc      0 for external negative input pin; 1 for internal band-gap voltage.
 *
 * @return     None
 *
 * @details    Select ACMP negative input source.
 */
static __INLINE void ACMP_SelNegInput(uint32_t u32AcmpModule, uint32_t u32NegSrc)
{
    if(u32AcmpModule == 0)
        ACMP01->CR0 = (ACMP01->CR0 & (~ACMP_CR_NEGSEL_Msk)) | (u32NegSrc << ACMP_CR_NEGSEL_Pos);
    else if(u32AcmpModule == 1)
        ACMP01->CR1 = (ACMP01->CR1 & (~ACMP_CR_NEGSEL_Msk)) | (u32NegSrc << ACMP_CR_NEGSEL_Pos);
    else if(u32AcmpModule == 2)
        ACMP23->CR2 = (ACMP23->CR2 & (~ACMP_CR_NEGSEL_Msk)) | (u32NegSrc << ACMP_CR_NEGSEL_Pos);
    else
        ACMP23->CR3 = (ACMP23->CR3 & (~ACMP_CR_NEGSEL_Msk)) | (u32NegSrc << ACMP_CR_NEGSEL_Pos);
}

/**
 * @brief      Enable ACMP hysteresis function
 *
 * @param[in]  u32AcmpModule  Specify the analog comparator module
 *
 * @return     None
 *
 * @details    Enable ACMP hysteresis function.
 */
static __INLINE void ACMP_EnableHys(uint32_t u32AcmpModule)
{
    if(u32AcmpModule == 0)
        ACMP01->CR0 |= ACMP_CR_HYSTERESIS_EN;
    else if(u32AcmpModule == 1)
        ACMP01->CR1 |= ACMP_CR_HYSTERESIS_EN;
    else if(u32AcmpModule == 2)
        ACMP23->CR2 |= ACMP_CR_HYSTERESIS_EN;
    else
        ACMP23->CR3 |= ACMP_CR_HYSTERESIS_EN;
}

/**
 * @brief      Disable ACMP hysteresis function
 *
 * @param[in]  u32AcmpModule  Specify the analog comparator module
 *
 * @return     None
 *
 * @details    Disable ACMP hysteresis function.
 */
static __INLINE void ACMP_DisableHys(uint32_t u32AcmpModule)
{
    if(u32AcmpModule == 0)
        ACMP01->CR0 &= (~ACMP_CR_HYSTERESIS_EN);
    else if(u32AcmpModule == 1)
        ACMP01->CR1 &= (~ACMP_CR_HYSTERESIS_EN);
    else if(u32AcmpModule == 2)
        ACMP23->CR2 &= (~ACMP_CR_HYSTERESIS_EN);
    else
        ACMP23->CR3 &= (~ACMP_CR_HYSTERESIS_EN);
}

/**
 * @brief      Enable ACMP interrupt function
 *
 * @param[in]  u32AcmpModule  Specify the analog comparator module
 *
 * @return     None
 *
 * @details    Enable ACMP interrupt function.
 */
static __INLINE void ACMP_EnableInt(uint32_t u32AcmpModule)
{
    if(u32AcmpModule == 0)
        ACMP01->CR0 |= ACMP_CR_INT_EN;
    else if(u32AcmpModule == 1)
        ACMP01->CR1 |= ACMP_CR_INT_EN;
    else if(u32AcmpModule == 2)
        ACMP23->CR2 |= ACMP_CR_INT_EN;
    else
        ACMP23->CR3 |= ACMP_CR_INT_EN;
}

/**
 * @brief      Disable ACMP interrupt function
 *
 * @param[in]  u32AcmpModule  Specify the analog comparator module
 *
 * @return     None
 *
 * @details    Disable ACMP interrupt function.
 */
static __INLINE void ACMP_DisableInt(uint32_t u32AcmpModule)
{
    if(u32AcmpModule == 0)
        ACMP01->CR0 &= (~ACMP_CR_INT_EN);
    else if(u32AcmpModule == 1)
        ACMP01->CR1 &= (~ACMP_CR_INT_EN);
    else if(u32AcmpModule == 2)
        ACMP23->CR2 &= (~ACMP_CR_INT_EN);
    else
        ACMP23->CR3 &= (~ACMP_CR_INT_EN);
}

/**
 * @brief      Enable ACMP
 *
 * @param[in]  u32AcmpModule  Specify the analog comparator module
 *
 * @return     None
 *
 * @details    Enable ACMP.
 */
static __INLINE void ACMP_Enable(uint32_t u32AcmpModule)
{
    if(u32AcmpModule == 0)
        ACMP01->CR0 |= ACMP_CR_ACMP_EN;
    else if(u32AcmpModule == 1)
        ACMP01->CR1 |= ACMP_CR_ACMP_EN;
    else if(u32AcmpModule == 2)
        ACMP23->CR2 |= ACMP_CR_ACMP_EN;
    else
        ACMP23->CR3 |= ACMP_CR_ACMP_EN;
}

/**
 * @brief      Disable ACMP
 *
 * @param[in]  u32AcmpModule  Specify the analog comparator module
 *
 * @return     None
 *
 * @details    Disable ACMP.
 */
static __INLINE void ACMP_Disable(uint32_t u32AcmpModule)
{
    if(u32AcmpModule == 0)
        ACMP01->CR0 &= (~ACMP_CR_ACMP_EN);
    else if(u32AcmpModule == 1)
        ACMP01->CR1 &= (~ACMP_CR_ACMP_EN);
    else if(u32AcmpModule == 2)
        ACMP23->CR2 &= (~ACMP_CR_ACMP_EN);
    else
        ACMP23->CR3 &= (~ACMP_CR_ACMP_EN);
}

/**
 * @brief      Return ACMP01 status
 *
 * @param[in]  u32Mask        Status mask
 *
 * @return     The ACMP01 status specified by u32Mask
 *
 * @details    Used to get the ACMP01 output status and interrupt status.
 */
static __INLINE uint32_t ACMP_GetAcmp01Status(uint32_t u32Mask)
{
    return (ACMP01->SR01 & u32Mask);
}

/**
 * @brief      Return ACMP23 status
 *
 * @param[in]  u32Mask        Status mask
 *
 * @return     The ACMP23 status specified by u32Mask
 *
 * @details    Used to get the ACMP23 output status and interrupt status.
 */
static __INLINE uint32_t ACMP_GetAcmp23Status(uint32_t u32Mask)
{
    return (ACMP23->SR23 & u32Mask);
}

/**
 * @brief      Clear ACMP01 interrupt status
 *
 * @param[in]  u32Mask        Status mask
 *
 * @return     None
 *
 * @details    Used to clear the ACMP01 interrupt status specified by u32Mask.
 */
static __INLINE void ACMP_ClrAcmp01IntFlag(uint32_t u32Mask)
{
    ACMP01->SR01 = u32Mask;
}

/**
 * @brief      Clear ACMP23 interrupt status
 *
 * @param[in]  u32Mask        Status mask
 *
 * @return     None
 *
 * @details    Used to clear the ACMP23 interrupt status specified by u32Mask.
 */
static __INLINE void ACMP_ClrAcmp23IntFlag(uint32_t u32Mask)
{
    ACMP23->SR23 = u32Mask;
}


/**
  * @} End of ACMP Device Function Interface
  */

/**
  * @} End of M051 Function Interface
  */


#endif

