/**************************************************************************//**
 * @file     main.c
 * @version  V2.10
 * $Revision: 4 $
 * $Date: 14/02/11 7:25p $
 * @brief    M051 Series Timer Controller Driver Sample Code
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "M051Series.h"

#define PLLCON_SETTING      SYSCLK_PLLCON_50MHz_XTAL
#define PLL_CLOCK           50000000


/*---------------------------------------------------------------------------------------------------------*/
/* Global Interface Variables Declarations                                                                 */
/*---------------------------------------------------------------------------------------------------------*/
extern int IsDebugFifoEmpty(void);
volatile uint8_t  g_u8IsWDTTimeoutINT = 0;
volatile uint32_t g_au32TMRINTCount[4] = {0};


/**
 * @brief       Timer-0 IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The TIMER0 default IRQ, declared in startup_M051Series.s.
 */
void TMR0_IRQHandler(void)
{
    /* Clear TIMER0 Time-out Interrupt Flag */
    _TIMER_CLEAR_CMP_INT_FLAG(TIMER0);

    g_au32TMRINTCount[0]++;
}

/**
 * @brief       Timer-1 IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The TIMER1 default IRQ, declared in startup_M051Series.s.
 */
void TMR1_IRQHandler(void)
{
    if(_TIMER_GET_CMP_INT_FLAG(TIMER1) == 1)
    {
        /* Clear TIMER1 Time-out Interrupt Flag */
        _TIMER_CLEAR_CMP_INT_FLAG(TIMER1);
    }
    else if(_TIMER_GET_CAP_INT_FLAG(TIMER1) == 1)
    {
        /* Clear TIMER1 Capture Interrupt Flag */
        _TIMER_CLEAR_CAP_INT_FLAG(TIMER1);
    }

    g_au32TMRINTCount[1]++;
}

/**
 * @brief       Timer-2 IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The TIMER2 default IRQ, declared in startup_M051Series.s.
 */
void TMR2_IRQHandler(void)
{
    /* Clear TIMER2 Time-out Interrupt Flag */
    _TIMER_CLEAR_CMP_INT_FLAG(TIMER2);

    g_au32TMRINTCount[2]++;
}

/**
 * @brief       Timer-3 IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The TIMER3 default IRQ, declared in startup_M051Series.s.
 */
void TMR3_IRQHandler(void)
{
    /* Clear TIMER3 Time-out Interrupt Flag */
    _TIMER_CLEAR_CMP_INT_FLAG(TIMER3);

    g_au32TMRINTCount[3]++;
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable IRC22M clock */
    SYSCLK->PWRCON |= SYSCLK_PWRCON_IRC22M_EN_Msk;

    /* Waiting for IRC22M clock ready */
    SYS_WaitingForClockReady(SYSCLK_CLKSTATUS_IRC22M_STB_Msk);

    /* Switch HCLK clock source to IRC22M */
    SYSCLK->CLKSEL0 = SYSCLK_CLKSEL0_HCLK_IRC22M;

    /* Set PLL to Power-down mode and PLL_STB bit in CLKSTATUS register will be cleared by hardware.*/
    SYSCLK->PLLCON |= SYSCLK_PLLCON_PD_Msk;

    /* Enable external 12 MHz XTAL, IRC10K */
    SYSCLK->PWRCON |= SYSCLK_PWRCON_XTL12M_EN_Msk | SYSCLK_PWRCON_OSC10K_EN_Msk;

    /* Enable PLL and Set PLL frequency */
    SYSCLK->PLLCON = PLLCON_SETTING;

    /* Waiting for clock ready */
    SYS_WaitingForClockReady(SYSCLK_CLKSTATUS_PLL_STB_Msk | SYSCLK_CLKSTATUS_XTL12M_STB_Msk | SYSCLK_CLKSTATUS_IRC10K_STB_Msk);

    /* Switch HCLK clock source to PLL, STCLK to HCLK/2 */
    SYSCLK->CLKSEL0 = SYSCLK_CLKSEL0_STCLK_HCLK_DIV2 | SYSCLK_CLKSEL0_HCLK_PLL;

    /* Enable peripheral clock */
    SYSCLK->APBCLK = SYSCLK_APBCLK_UART0_EN_Msk | SYSCLK_APBCLK_WDT_EN_Msk |
                     SYSCLK_APBCLK_TMR0_EN_Msk | SYSCLK_APBCLK_TMR1_EN_Msk | SYSCLK_APBCLK_TMR2_EN_Msk | SYSCLK_APBCLK_TMR3_EN_Msk;

    /* Peripheral clock source */
    SYSCLK->CLKSEL1 = SYSCLK_CLKSEL1_UART_PLL | SYSCLK_CLKSEL1_WDT_IRC10K |
                      SYSCLK_CLKSEL1_TMR0_XTAL | SYSCLK_CLKSEL1_TMR1_HCLK | SYSCLK_CLKSEL1_TMR2_IRC22M | SYSCLK_CLKSEL1_TMR3_XTAL;

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    //SystemCoreClockUpdate();
    PllClock        = PLL_CLOCK;            // PLL
    SystemCoreClock = PLL_CLOCK / 1;        // HCLK
    CyclesPerUs     = PLL_CLOCK / 1000000;  // For SYS_SysTickDelay()

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set P3 multi-function pins for UART0 RXD, TXD, T0, T1 and T1EX */
    SYS->P3_MFP = SYS_MFP_P30_RXD0 | SYS_MFP_P31_TXD0 | SYS_MFP_P34_T0 | SYS_MFP_P35_T1 | SYS_MFP_P33_T1EX;

    /* Set P1 multi-function pins for T3 */
    SYS->P1_MFP = SYS_MFP_P11_T3;

    /* Lock protected registers */
    SYS_LockReg();
}

void UART0_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART */
    SYS->IPRSTC2 |=  SYS_IPRSTC2_UART0_RST_Msk;
    SYS->IPRSTC2 &= ~SYS_IPRSTC2_UART0_RST_Msk;

    /* Configure UART0 and set UART0 Baudrate */
    UART0->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(PLL_CLOCK, 115200);
    _UART_SET_DATA_FORMAT(UART0, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1);
}

/*---------------------------------------------------------------------------------------------------------*/
/*  MAIN function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int main(void)
{
    volatile uint32_t u32InitCount;

    /* Init System, peripheral clock and multi-function I/O */
    SYS_Init();

    /* Init UART0 for printf */
    UART0_Init();

    printf("\n\nCPU @ %d Hz\n", SystemCoreClock);
    printf("+---------------------------------------------------+\n");
    printf("|    TIMER External Capture Function Sample Code    |\n");
    printf("+---------------------------------------------------+\n\n");

    printf("# TIMER Settings:\n");
    printf("  TIMER0: Clock source is 12 MHz; Toggle-Output mode and frequency is 500 Hz; Disable interrupt.\n");
    printf("  TIMER3: Clock source is 12 MHz; Toggle-Output mode and frequency is 1 Hz; Disable interrupt.\n");
    printf("  TIMER1: Clock source is HCLK(48 MHz); Periodic mode; Disable interrupt; Pre-scale 1; TCMP is 0xFFFFFF;\n");
    printf("          Counter pin enable; Capture pin enable;\n");
    printf("# Generate 500 Hz frequency of T0 and connect T0 to T1 counter pin.\n");
    printf("# Generate 1 Hz frequency of T3 and connect T3 to T1EX capture pin.\n");
    printf("# Get 500 counts from T1 pin when each T1EX pin interrupt occurred.\n\n");

    /* Reset and stop TIMER0, TIMER1 and TIMER3 counting first */
    _TIMER_RESET(TIMER0);
    _TIMER_RESET(TIMER1);
    _TIMER_RESET(TIMER3);

    /* Enable TIMER1 NVIC for T1EX interrupt */
    NVIC_EnableIRQ(TMR1_IRQn);

    /* Configure TCMP values of TIMER0 and TIMER3 */
    TIMER0->TCMPR = (__XTAL / 1000); // For 1000 Hz (Equal to 500Hz toggle rate)
    TIMER3->TCMPR = __XTAL / 2;     // For 2 Hz (Prescale = 1)

    /* Start TIMER0 counting and output T0 frequency to 500 Hz*/
    TIMER0->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_MODE_TOGGLE | TIMER_TCSR_TDR_EN_Msk | TIMER_TCSR_PRESCALE(1);

    /* Start TIMER3 counting and output T3 frequency to 1 Hz */
    TIMER3->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_MODE_TOGGLE | TIMER_TCSR_TDR_EN_Msk | TIMER_TCSR_PRESCALE(1);

    /* Enable TIMER1 counter input and capture function */
    TIMER1->TCMPR = 0xFFFFFF;
    TIMER1->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_IE_Msk | TIMER_TCSR_MODE_CONTINUOUS | TIMER_TCSR_CTB_ENABLE |
                   TIMER_TCSR_TDR_EN_Msk | TIMER_TCSR_PRESCALE(1);
    TIMER1->TEXCON = TIMER_TEXCON_MODE_CAP | TIMER_TEXCON_TEXIEN_ENABLE | TIMER_TEXCON_TEXEN_ENABLE;

    /* Check T1EX interrupt counts */
    u32InitCount = g_au32TMRINTCount[1] = 0;
    while(g_au32TMRINTCount[1] <= 10)
    {
        if(g_au32TMRINTCount[1] != u32InitCount)
        {
            printf("[%d]- %d\n", g_au32TMRINTCount[1], _TIMER_GET_TCAP(TIMER1));
            u32InitCount = g_au32TMRINTCount[1];
        }
    }

    /* Reset and stop TIMER0, TIMER1 and TIMER3 counting */
    _TIMER_RESET(TIMER0);
    _TIMER_RESET(TIMER1);
    _TIMER_RESET(TIMER3);

    printf("PASS.\n");

    while(1);
}
