/**************************************************************************//**
 * @file     main.c
 * @version  V2.10
 * $Revision: 3 $
 * $Date: 14/02/11 7:25p $
 * @brief    M051 Series Timer Controller Driver Sample Code
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "M051Series.h"

#define PLLCON_SETTING      SYSCLK_PLLCON_50MHz_XTAL
#define PLL_CLOCK           50000000


/*---------------------------------------------------------------------------------------------------------*/
/* Global Interface Variables Declarations                                                                 */
/*---------------------------------------------------------------------------------------------------------*/
volatile uint32_t g_au32TMRINTCount[4] = {0};


/**
 * @brief       Timer-0 IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The TIMER0 default IRQ, declared in startup_M051Series.s.
 */
void TMR0_IRQHandler(void)
{
    /* Clear TIMER0 Time-out Interrupt Flag */
    _TIMER_CLEAR_CMP_INT_FLAG(TIMER0);

    g_au32TMRINTCount[0]++;
}

/**
 * @brief       Timer-1 IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The TIMER1 default IRQ, declared in startup_M051Series.s.
 */
void TMR1_IRQHandler(void)
{
    /* Clear TIMER1 Time-out Interrupt Flag */
    _TIMER_CLEAR_CMP_INT_FLAG(TIMER1);

    g_au32TMRINTCount[1]++;
}

/**
 * @brief       Timer-2 IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The TIMER2 default IRQ, declared in startup_M051Series.s.
 */
void TMR2_IRQHandler(void)
{
    /* Clear TIMER2 Time-out Interrupt Flag */
    _TIMER_CLEAR_CMP_INT_FLAG(TIMER2);

    g_au32TMRINTCount[2]++;
}

/**
 * @brief       Timer-3 IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The TIMER3 default IRQ, declared in startup_M051Series.s.
 */
void TMR3_IRQHandler(void)
{
    /* Clear TIMER3 Time-out Interrupt Flag */
    _TIMER_CLEAR_CMP_INT_FLAG(TIMER3);

    g_au32TMRINTCount[3]++;
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable IRC22M clock */
    SYSCLK->PWRCON |= SYSCLK_PWRCON_IRC22M_EN_Msk;

    /* Waiting for IRC22M clock ready */
    SYS_WaitingForClockReady(SYSCLK_CLKSTATUS_IRC22M_STB_Msk);

    /* Switch HCLK clock source to IRC22M */
    SYSCLK->CLKSEL0 = SYSCLK_CLKSEL0_HCLK_IRC22M;

    /* Set PLL to Power-down mode and PLL_STB bit in CLKSTATUS register will be cleared by hardware.*/
    SYSCLK->PLLCON |= SYSCLK_PLLCON_PD_Msk;

    /* Enable external 12 MHz XTAL, IRC10K */
    SYSCLK->PWRCON |= SYSCLK_PWRCON_XTL12M_EN_Msk | SYSCLK_PWRCON_OSC10K_EN_Msk;

    /* Enable PLL and Set PLL frequency */
    SYSCLK->PLLCON = PLLCON_SETTING;

    /* Waiting for clock ready */
    SYS_WaitingForClockReady(SYSCLK_CLKSTATUS_PLL_STB_Msk | SYSCLK_CLKSTATUS_XTL12M_STB_Msk | SYSCLK_CLKSTATUS_IRC10K_STB_Msk);

    /* Switch HCLK clock source to PLL, STCLK to HCLK/2 */
    SYSCLK->CLKSEL0 = SYSCLK_CLKSEL0_STCLK_HCLK_DIV2 | SYSCLK_CLKSEL0_HCLK_PLL;

    /* Enable peripheral clock */
    SYSCLK->APBCLK = SYSCLK_APBCLK_UART0_EN_Msk | SYSCLK_APBCLK_WDT_EN_Msk |
                     SYSCLK_APBCLK_TMR0_EN_Msk | SYSCLK_APBCLK_TMR1_EN_Msk | SYSCLK_APBCLK_TMR2_EN_Msk | SYSCLK_APBCLK_TMR3_EN_Msk;

    /* Peripheral clock source */
    SYSCLK->CLKSEL1 = SYSCLK_CLKSEL1_UART_PLL | SYSCLK_CLKSEL1_WDT_IRC10K |
                      SYSCLK_CLKSEL1_TMR0_XTAL | SYSCLK_CLKSEL1_TMR1_HCLK | SYSCLK_CLKSEL1_TMR2_IRC22M | SYSCLK_CLKSEL1_TMR3_XTAL;

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    //SystemCoreClockUpdate();
    PllClock        = PLL_CLOCK;            // PLL
    SystemCoreClock = PLL_CLOCK / 1;        // HCLK
    CyclesPerUs     = PLL_CLOCK / 1000000;  // For SYS_SysTickDelay()

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set P3 multi-function pins for UART0 RXD, TXD */
    SYS->P3_MFP = SYS_MFP_P30_RXD0 | SYS_MFP_P31_TXD0;

    /* Lock protected registers */
    SYS_LockReg();
}

void UART0_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART */
    SYS->IPRSTC2 |=  SYS_IPRSTC2_UART0_RST_Msk;
    SYS->IPRSTC2 &= ~SYS_IPRSTC2_UART0_RST_Msk;

    /* Configure UART0 and set UART0 Baudrate */
    UART0->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(PLL_CLOCK, 115200);
    _UART_SET_DATA_FORMAT(UART0, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1);
}

/*---------------------------------------------------------------------------------------------------------*/
/*  MAIN function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int main(void)
{
    volatile uint32_t u32InitCount;

    /* Init System, peripheral clock and multi-function I/O */
    SYS_Init();

    /* Init UART0 for printf */
    UART0_Init();

    printf("\n\nCPU @ %d Hz\n", SystemCoreClock);
    printf("+------------------------------------------------+\n");
    printf("|    TIMER Periodic and Interrupt Sample Code    |\n");
    printf("+------------------------------------------------+\n\n");

    printf("# TIMER Settings:\n");
    printf("  TIMER0: Clock source 12 MHz;     Periodic mode; Enable interrupt; 1 interrupt tick/sec.\n");
    printf("  TIMER1: Clock source 48 MHz;     Periodic mode; Enable interrupt; 2 interrupt ticks/sec.\n");
    printf("  TIMER2: Clock source IRC 22 MHz; Periodic mode; Enable interrupt; 4 interrupt ticks/sec.\n");
    printf("  TIMER3: Clock source 12 MHz;     Periodic mode; Enable interrupt; 8 interrupt ticks/sec.\n");
    printf("# Check all Timer interrupt counts are reasonable or not.\n");

    /* Reset and stop TIMER0 ~ TIMER3 counting first */
    _TIMER_RESET(TIMER0);
    _TIMER_RESET(TIMER1);
    _TIMER_RESET(TIMER2);
    _TIMER_RESET(TIMER3);

    /* Enable TIMER0 ~ TIMER3 NVIC */
    NVIC_EnableIRQ(TMR0_IRQn);
    NVIC_EnableIRQ(TMR1_IRQn);
    NVIC_EnableIRQ(TMR2_IRQn);
    NVIC_EnableIRQ(TMR3_IRQn);

    /* To Configure TCMPR values based on Timer clock source and pre-scale value */
    TIMER0->TCMPR = __XTAL;                  // For 1 tick per second when using external 12 MHz (Prescale = 1)
    TIMER1->TCMPR = ((SystemCoreClock / 4) / 2); // For 2 ticks per second when using 48 MHz HCLK (Prescale = 4)
    TIMER2->TCMPR = ((__IRC22M / 1) / 4);    // For 4 ticks per second when using IRC 22.1184 MHz (Prescale = 1)
    TIMER3->TCMPR = ((__XTAL / 1) / 8);      // For 8 ticks per second when using external 12 MHz (Prescale = 1)

    /* Clear Timer interrupt counts to 0 */
    g_au32TMRINTCount[0] = g_au32TMRINTCount[1] = g_au32TMRINTCount[2] = g_au32TMRINTCount[3] = 0;
    u32InitCount = g_au32TMRINTCount[0];

    /* Start TIMER0 counting */
    TIMER0->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_IE_Msk | TIMER_TCSR_MODE_PERIODIC | TIMER_TCSR_TDR_EN_Msk |
                   TIMER_TCSR_PRESCALE(1);

    /* Start TIMER1 counting */
    TIMER1->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_IE_Msk | TIMER_TCSR_MODE_PERIODIC | TIMER_TCSR_TDR_EN_Msk |
                   TIMER_TCSR_PRESCALE(4);

    /* Start TIMER2 counting */
    TIMER2->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_IE_Msk | TIMER_TCSR_MODE_PERIODIC | TIMER_TCSR_TDR_EN_Msk |
                   TIMER_TCSR_PRESCALE(1);

    /* Start TIMER3 counting */
    TIMER3->TCSR = TIMER_TCSR_CEN_Msk | TIMER_TCSR_IE_Msk | TIMER_TCSR_MODE_PERIODIC | TIMER_TCSR_TDR_EN_Msk |
                   TIMER_TCSR_PRESCALE(1);

    /* Check Timer interrupt counts */
    printf("# Timer interrupt counts :\n");
    while(u32InitCount < 20)
    {
        if(g_au32TMRINTCount[0] != u32InitCount)
        {
            printf("TMR0:%d     TMR1:%d     TMR2:%d     TMR3:%d\n",
                   g_au32TMRINTCount[0], g_au32TMRINTCount[1], g_au32TMRINTCount[2], g_au32TMRINTCount[3]);
            u32InitCount = g_au32TMRINTCount[0];
        }
    }
    printf("*** PASS ***\n");

    while(1);
}
